@@ -1,4 +1,4 @@
-//! Adapted from Clippy's driver: https://github.com/rust-lang/rust-clippy/blob/fd30241281333d73d504355b2f4d0ecd94f27b0e/src/driver.rs
+//! Adapted from [Clippy's driver](https://github.com/rust-lang/rust-clippy/blob/fd30241281333d73d504355b2f4d0ecd94f27b0e/src/driver.rs)
 
 #![feature(rustc_private)]
 
@@ -18,16 +18,13 @@ use std::{
 
 use flowistry::{
   extensions::{ContextMode, EvalMode, MutabilityMode, PointerMode, EVAL_MODE},
-  infoflow::Direction,
-};
-use flowistry_ide::{
-  analysis::{FlowistryError, FlowistryResult},
   range::{FunctionIdentifier, Range},
 };
+use flowistry_ide::{FlowistryError, FlowistryResult, JsonEncodable};
 use fluid_let::fluid_set;
 use log::debug;
 use rustc_interface::interface::Result as RustcResult;
-use rustc_serialize::{json, Encodable};
+use rustc_serialize::json;
 
 fn arg<T>(s: &str) -> T
 where
@@ -73,10 +70,8 @@ fn toolchain_path(home: Option<String>, toolchain: Option<String>) -> Option<Pat
   })
 }
 
-fn try_analysis<T: for<'a> Encodable<json::Encoder<'a>>>(
-  f: impl FnOnce() -> FlowistryResult<T>,
-) -> RustcResult<()> {
-  let result = match f() {
+fn postprocess<T: JsonEncodable>(result: FlowistryResult<T>) -> RustcResult<()> {
+  let result = match result {
     Ok(output) => Ok(output),
     Err(e) => match e {
       FlowistryError::BuildError => {
@@ -94,55 +89,35 @@ fn try_analysis<T: for<'a> Encodable<json::Encoder<'a>>>(
 fn run_flowistry(args: &[String]) -> RustcResult<()> {
   debug!("Running flowistry with args: {}", args.join(" "));
 
+  let context_mode = match arg::<String>("CONTEXT_MODE").as_str() {
+    "Recurse" => ContextMode::Recurse,
+    "SigOnly" => ContextMode::SigOnly,
+    flag => panic!("Bad value of context mode: {flag}"),
+  };
+
+  let mutability_mode = match arg::<String>("MUTABILITY_MODE").as_str() {
+    "DistinguishMut" => MutabilityMode::DistinguishMut,
+    "IgnoreMut" => MutabilityMode::IgnoreMut,
+    flag => panic!("Bad value of context mode: {flag}"),
+  };
+
+  let pointer_mode = match arg::<String>("POINTER_MODE").as_str() {
+    "Precise" => PointerMode::Precise,
+    "Conservative" => PointerMode::Conservative,
+    flag => panic!("Bad value of pointer mode: {flag}"),
+  };
+
+  let eval_mode = EvalMode {
+    context_mode,
+    mutability_mode,
+    pointer_mode,
+  };
+  fluid_set!(EVAL_MODE, eval_mode);
+
   match arg::<String>("COMMAND").as_str() {
-    cmd @ ("backward_slice" | "forward_slice") => {
-      let range = Range {
-        start: arg::<usize>("START"),
-        end: arg::<usize>("END"),
-        filename: arg::<String>("FILE"),
-      };
-
-      let direction = if cmd == "backward_slice" {
-        Direction::Backward
-      } else {
-        Direction::Forward
-      };
-
-      let context_mode = match arg::<String>("CONTEXT_MODE").as_str() {
-        "Recurse" => ContextMode::Recurse,
-        "SigOnly" => ContextMode::SigOnly,
-        flag => panic!("Bad value of context mode: {flag}"),
-      };
-
-      let mutability_mode = match arg::<String>("MUTABILITY_MODE").as_str() {
-        "DistinguishMut" => MutabilityMode::DistinguishMut,
-        "IgnoreMut" => MutabilityMode::IgnoreMut,
-        flag => panic!("Bad value of context mode: {flag}"),
-      };
-
-      let pointer_mode = match arg::<String>("POINTER_MODE").as_str() {
-        "Precise" => PointerMode::Precise,
-        "Conservative" => PointerMode::Conservative,
-        flag => panic!("Bad value of pointer mode: {flag}"),
-      };
-
-      let eval_mode = EvalMode {
-        context_mode,
-        mutability_mode,
-        pointer_mode,
-      };
-      fluid_set!(EVAL_MODE, eval_mode);
-
-      try_analysis(move || flowistry_ide::slicing::slice(direction, range, args))
-    }
-    "find_mutations" => {
-      let range = Range {
-        start: arg::<usize>("START"),
-        end: arg::<usize>("END"),
-        filename: arg::<String>("FILE"),
-      };
-
-      try_analysis(move || flowistry_ide::mutations::find(range, args))
+    "spans" => {
+      let filename = arg::<String>("FILE");
+      postprocess(flowistry_ide::spans::spans(args, filename))
     }
     "playground" => {
       let range = Range {
@@ -151,9 +126,13 @@ fn run_flowistry(args: &[String]) -> RustcResult<()> {
         filename: arg::<String>("FILE"),
       };
 
-      try_analysis(move || flowistry_ide::playground::playground(range, args))
+      postprocess(flowistry_ide::run(
+        flowistry_ide::playground::playground,
+        range,
+        args,
+      ))
     }
-    cmd @ ("effects" | "decompose" | "focus") => {
+    cmd @ ("decompose" | "focus") => {
       let pos = arg::<usize>("POS");
       let id = FunctionIdentifier::Range(Range {
         start: pos,
@@ -161,11 +140,12 @@ fn run_flowistry(args: &[String]) -> RustcResult<()> {
         filename: arg::<String>("FILE"),
       });
       match cmd {
-        "effects" => try_analysis(move || flowistry_ide::effects::effects(id, args)),
-        "decompose" => {
-          try_analysis(move || flowistry_ide::decompose::decompose(id, args))
-        }
-        "focus" => try_analysis(move || flowistry_ide::focus::focus(id, args)),
+        "decompose" => postprocess(flowistry_ide::run(
+          flowistry_ide::decompose::decompose,
+          id,
+          args,
+        )),
+        "focus" => postprocess(flowistry_ide::run(flowistry_ide::focus::focus, id, args)),
         _ => unreachable!(),
       }
     }
